package org.ow2.odis.core;

import java.util.Observable;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.lifeCycle.connection.AbstractConnectionState;
import org.ow2.odis.lifeCycle.connection.IConnectionLifeCycle;
import org.ow2.odis.lifeCycle.connection.StateConnectionLoaded;
import org.ow2.odis.exception.OdisAttributeException;
import org.ow2.odis.exception.OdisConnectionException;
import org.ow2.odis.model.AbstractConnectionAttribute;


/**
 * The Class AbstractSqlConnection.
 */
public abstract class AbstractConnection implements IConnectionLifeCycle {
    /** logger of this class. */
    static final Logger logger = Monolog.initialize().getLogger(
            AbstractConnection.class.getName());

    /** The message sender. */
    protected IMessageSender messageSender = null;

    /** The running. */
    protected boolean running = true;

    /** attribute with contain all properties of the connection. */
    public final AConnectionAttribute attribute;

    /** connection state. */
    private AbstractConnectionState cnxState = new StateConnectionLoaded();

    /** The connection. */
    public IConnection cnx = null;

    /** The Sql server. */
    private Thread sqlServer;

    /**
     * Constructor.
     * @param attribute the attribute
     */
    AbstractConnection(AConnectionAttribute attribute) {
        super();
        this.attribute = attribute;
    }

    /**
     * Checks if is running.
     * @return true, if is running
     */
    public boolean isRunning() {
        return running;
    }

    /**
     * Sets the running.
     * @param running the new running
     */
    public void setRunning(boolean running) {
        this.running = running;
    }

    /**
     * Inits the component.
     * @throws OdisConnectionException 
     * @see com.francetelecom.odis.component.lifeCycle.connexion.ICnxLifeCycle#initComponent()
     */
    public void initComponent() throws OdisConnectionException {
        try {
            attribute.checkProperties();
        } catch (OdisAttributeException e) {
            throw new OdisConnectionException(e);
        }
    }

    /**
     * Sets the cnx state.
     * @param state the state
     */
    public void setConnectionState(AbstractConnectionState state) {
        this.cnxState = state;
        if (logger.isLoggable(BasicLevel.INFO)) {
            StringBuffer buffer = new StringBuffer(toString());
            buffer.append(" set to ");
            buffer.append(state.getStateName());
            logger.log(BasicLevel.INFO, buffer.toString());
        }
    }

    /**
     * Gets the cnx state.
     * @return the cnx state
     * @see com.francetelecom.odis.component.lifeCycle.connexion.ICnxLifeCycle#getConnectionState()
     */
    public AbstractConnectionState getConnectionState() {
        return cnxState;
    }

    /**
     * Update.
     * @param o the o
     * @param arg the arg
     * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
     */
    public void update(Observable o, Object arg) {
    }

    /**
     * Gets the attribut.
     * @return the attribut
     * @see com.francetelecom.odis.component.lifeCycle.connexion.ICnxLifeCycle#getAttribute()
     */
    public AbstractConnectionAttribute getAttribute() {
        return attribute;
    }

    /**
     * Launch component.
     * @throws OdisConnectionException the odis cnx exception
     * @see com.francetelecom.odis.component.lifeCycle.connexion.ICnxLifeCycle#launchComponent()
     */
    public void launchComponent() throws OdisConnectionException {
        AbstractKeepConnexion connexion = getKeepConnection();
        this.cnx = connexion.getConnection();
        sqlServer = new Thread(connexion, attribute.getTypeCnx() + "_Connexion");
        sqlServer.start();
    }

    /**
     * Gets the keep connection.
     * @return the keep connection
     */
    protected abstract AbstractKeepConnexion getKeepConnection();

    /**
     * Stop component.
     * @see com.francetelecom.odis.component.lifeCycle.connexion.ICnxLifeCycle#stopComponent()
     */
    public void stopComponent() {
        running = false;
        sqlServer.interrupt();
        if (cnx != null) {
            cnx.close();
        }
    }

    /**
     * Gets the message sender.
     * @return the message sender
     */
    public IMessageSender getMessageSender() {
        return messageSender;
    }

    /**
     * Sets the message sender.
     * @param messageSender the new message sender
     */
    public void setMessageSender(IMessageSender messageSender) {
        this.messageSender = messageSender;
    }

    /**
     * Sets the connection. Used in JUnit tests
     * @param cnx the new connection
     */
    public void setConnection(IConnection cnx) {
        this.cnx = cnx;
    }
}
