package org.ow2.odis.core;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.connection.IConnectionIn;
import org.ow2.odis.connection.context.Context;
import org.ow2.odis.connection.context.ContextMessage;
import org.ow2.odis.model.Const;
import org.ow2.odis.port.PortIn;


/**
 * base class of RXTXClientConnexionIn and RXTXServerConnexionIn.
 * @author Bogaert Sandra FranceTelecom R&D
 */
public abstract class AbstractConnectionIn extends AbstractConnection implements
        IConnectionIn {
    /** logger of this class. */
    static final Logger logger = Monolog.initialize().getLogger(
            AbstractConnectionIn.class.getName());

    /** define where the connection get the message. */
    protected PortIn portIn = null;

    /** The list of context messages. */
    private List lsContextMessages = Collections
            .synchronizedList(new ArrayList(5000));

    /** The connection. */
    protected AbstractConnectionIn connexionIn = null;

    /**
     * Constructor.
     * @param attribut the attribute
     */
    public AbstractConnectionIn(AConnectionAttribute attribut) {
        super(attribut);
        connexionIn = this;
    }

    /**
     * Wait until be able to add new message and then add it.
     * @param bytes array
     * @param context the context
     * @throws InterruptedException on a user request an Interrupted Exception can occurred to break the pause
     */
    public void addMessage(byte[] bytes, Context context)
            throws InterruptedException {
        if (!checkState(bytes.length)) {
            pause();
        }
        attribute.setState(Const.READY);
        final ContextMessage contextMessage = new ContextMessage(bytes, context);
        lsContextMessages.add(contextMessage);
        if (portIn != null) {
            portIn.onMessage();
        }
    }

    /**
     * use by send to freeze the thread send while the.
     * @throws InterruptedException the interrupted exception
     */
    private void pause() throws InterruptedException {
        if (attribute.getState() == Const.NOT_READY) {
            synchronized (this) {
                if (!checkStateAgain()) {
                    if (logger.isLoggable(BasicLevel.INFO)) {
                        logger.log(BasicLevel.INFO,
                                "Buffer full : entering wait() for " + this);
                    }
                    wait();
                    if (logger.isLoggable(BasicLevel.INFO)) {
                        logger.log(BasicLevel.INFO,
                                "Buffer available again : wait() exited for "
                                        + this);
                    }
                }
            }
        }
    }

    /**
     * check if all condition are respected if a new message with length byte is added.
     * @param length int size of message added
     * @return boolean
     */
    private boolean checkState(int length) {
        int nbMsgPresent = lsContextMessages.size();
        if (attribute.getSizeObjectMax() > 0
                && attribute.getSizeObjectMax() < nbMsgPresent) {
            if (logger.isLoggable(BasicLevel.WARN)) {
                StringBuffer errMsg = new StringBuffer(attribute.getTypeCnx());
                errMsg.append(" [");
                errMsg.append(attribute.getName());
                errMsg
                        .append("] refusing to addMessage() : too many messages present (");
                errMsg.append(nbMsgPresent);
                errMsg
                        .append(")\n\twhile the cnx IN buffer size has been set to : ");
                errMsg.append(attribute.getSizeObjectMax());
                logger.log(BasicLevel.WARN, errMsg.toString());
            }
            attribute.setState(Const.NOT_READY);
            return false;
        }
        if (attribute.getSizeByteMax() > 0
                && (attribute.getSizeByteMax()) < length
                        + attribute.getSizeByte()) {
            if (logger.isLoggable(BasicLevel.WARN)) {
                StringBuffer errMsg = new StringBuffer(attribute.getTypeCnx());
                errMsg.append(" [");
                errMsg.append(attribute.getName());
                errMsg
                        .append("] refusing to addMessage() : cnx IN buffer full");
                errMsg.append("\n\t->capacity (bytes) : ");
                errMsg.append(attribute.getSizeByteMax());
                errMsg.append("\n\t->current size     : ");
                errMsg.append(attribute.getSizeByte());
                errMsg.append(" (on ");
                errMsg.append(nbMsgPresent);
                errMsg.append(" message(s))");
                errMsg.append("\n\t->trying to add    : ");
                errMsg.append(length);
                errMsg.append(" bytes");
                logger.log(BasicLevel.WARN, errMsg.toString());
            }
            attribute.setState(Const.NOT_READY);
            return false;
        }
        return true;
    }

    /**
     * Check state again.
     * @return true if only half buffer is at least free
     */
    private boolean checkStateAgain() {
        boolean result = true;
        result = (attribute.getSizeByteMax() < attribute.getSizeByte() * 2);
        result = result
                && (attribute.getSizeObjectMax()) < lsContextMessages.size() * 2;
        return result;
    }

    /**
     * Gets the pending message.
     * @return the pending message
     * @see com.francetelecom.odis.component.connexion.IConnexionIn#getPendingMessage()
     */
    public int getPendingMessage() {
        return lsContextMessages.size();
    }

    /**
     * Ack message.
     * @param valid the valid
     * @see com.francetelecom.odis.component.connexion.IConnexionIn#ackMessage(boolean)
     */
    public void ackMessage(boolean valid) {
        if (valid) {
            final ContextMessage contextMessage = (ContextMessage) lsContextMessages
                    .remove(0);
            attribute.incSizeByte(-contextMessage.getPayload().length);
            attribute.incNumberMessage(-1);
        }
        synchronized (this) {
            if (logger.isLoggable(BasicLevel.INFO)) {
                logger.log(BasicLevel.INFO,
                        "Buffer available again : notifying " + this);
            }
            notifyAll();
        }
    }

    /**
     * Gets the context message.
     * @return the context message
     * @see com.francetelecom.odis.component.connexion.IConnexionIn#getContextMessage()
     */
    public ContextMessage getContextMessage() {
        ContextMessage result = null;
        if (lsContextMessages.isEmpty() != true) {
            result = (ContextMessage) lsContextMessages.get(0);
        }
        return result;
    }

    /**
     * Sets the port in.
     * @param portIn the port in
     * @return the i connection in
     */
    public IConnectionIn setPortIn(PortIn portIn) {
        this.portIn = portIn;
        return this;
    }

    /**
     * Unload component.
     */
    public void unloadComponent() {
        attribute.deleteObserver(this);
    }
}
