package org.ow2.odis.core;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.model.AbstractConnectionAttribute;


/**
 * Create a message listener to look for message on this serial port. <br>
 * If the connection is broken it will recreate a new one.
 * @author Sandra Bogaert FranceTelecom R&D
 */
public abstract class AbstractKeepConnexion implements Runnable {
    /** The Constant LOGGER. */
    static final Logger LOGGER = Monolog.initialize().getLogger(
            AbstractKeepConnexion.class.getName());

    /** time (ms) before checking socket connection. */
    private static final int CHECK_CNX_DELAY = 1000;

    /** time before try to restore connection. */
    private static final int DELAY_BEFORE_RESTORE_CNX = 5000;

    /** The connection. */
    protected AbstractConnection connection = null;

    /** The connection. */
    protected IConnection cnx = null;

    /** The attribute. */
    protected AbstractConnectionAttribute attribute = null;

    /**
     * Instantiates a new RXTX keep connection.
     * @param connexion the connection
     * @param attribute the attribute
     */
    public AbstractKeepConnexion(AbstractConnection connexion,
            AbstractConnectionAttribute attribute) {
        super();
        this.connection = connexion;
        this.attribute = attribute;
        ThreadManager.getInstance().add(this);
        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
            LOGGER.log(BasicLevel.DEBUG, "Create " + attribute.getTypeConnection()
                    + " connection");
        }
    }

    /**
     * Run.
     * @see java.lang.Runnable#run()
     */
    public void run() {
        try {
            this.connection.cnx.open(attribute);
            MessageProcessor processor = getMessageProcessor(cnx);
            processor.setProtocol(getProtocol());
            while (connection.isRunning()) {
                if (processor instanceof IMessageSender) {
                    connection.setMessageSender((IMessageSender) processor);
                } else if (processor instanceof IMessageListener) {
                    ((IMessageListener) processor).start();
                }
                while (connection.isRunning() && !processor.stopProcessing) {
                    Thread.sleep(CHECK_CNX_DELAY);
                }
                if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                    LOGGER.log(BasicLevel.DEBUG, "the connexion is closed");
                }
                if (connection.isRunning()) {
                    try {
                        Thread.sleep(DELAY_BEFORE_RESTORE_CNX);
                    } catch (InterruptedException e) {
                        connection.setRunning(false);
                    }
                }
            }
            this.connection.cnx.close();
	        processor.stopProcessing = true;
        } catch (InterruptedException ex) {
            connection.setRunning(false);
        }
    }

    /**
     * Gets the message processor.
     * @param serialPort the serial port
     * @return the message processor (listener or sender)
     */
    protected abstract MessageProcessor getMessageProcessor(IConnection cnx);

    /**
     * Gets the connection.
     * @return the connection
     */
    protected abstract IConnection getConnection();

    /**
     * Gets the protocol.
     * @return the protocol
     */
    protected abstract IProtocol getProtocol();

    /**
     * Gets the serial port.
     * @return the serial port
     */
    public IConnection getCnx() {
        return cnx;
    }

    /**
     * Stop.
     */
    public void stop() {
    	this.connection.setRunning(false);
    }
}
