package org.ow2.odis.sql;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.exception.OdisAttributeException;
import org.ow2.odis.exception.OdisException;


public abstract class SqlConnectionHelper {
    /** logger of this class. */
    static final Logger LOGGER = Monolog.initialize().getLogger(
            SqlConnectionHelper.class.getName());

    /** The list prepared statement. */
    protected static Map listPreparedStatement = new HashMap();
    
    /**
     * Execute query.
     * @param query the query
     * @return the result set
     * @throws OdisAttributException the odis attribut exception
     * @throws SQLException the SQL exception
     */
    public static ResultSet executeQuery(String query)
            throws OdisAttributeException, SQLException {
        return executeQuery(query, null);
    }

    /**
     * Execute query.
     * @param query the query
     * @param parameters the parameters
     * @return the result set
     * @throws OdisAttributException the odis attribute exception
     * @throws SQLException the SQL exception
     */
    public static ResultSet executeQuery(String query,
            StatementParameterBean[] parameters) throws OdisAttributeException,
            SQLException {
        ResultSet result = null;
        
        Connection conn = null;
        try {
            conn = SqlConnectionHandler.getConnection();
            result = executeQuery(conn, query, parameters);
        } catch (OdisAttributeException e) {
            throw e;
        } catch (OdisException e) {
            if (LOGGER.isLoggable(BasicLevel.ERROR)) {
                LOGGER.log(BasicLevel.ERROR, e.getMessage());
            }
        } catch (SQLException e) {
            throw e;
        } finally {
            try {
                SqlConnectionHandler.freeConnection(conn);
            } catch (Exception e) {
            }
        }
        return result;
    }

    /**
     * Execute.
     * @param query the query
     * @throws OdisAttributException the odis attribut exception
     * @throws SQLException the SQL exception
     */
    public static void execute(String query) throws OdisAttributeException,
            SQLException {
        execute(query, null);
    }

    /**
     * Execute.
     * @param query the query
     * @param parameters the parameters
     * @throws OdisAttributException the odis attribut exception
     * @throws SQLException the SQL exception
     */
    public static void execute(String query, StatementParameterBean[] parameters)
            throws OdisAttributeException, SQLException {
        Connection conn = null;
        try {
            conn = SqlConnectionHandler.getConnection();
            execute(conn, query, parameters);
        } catch (OdisAttributeException e) {
            throw e;
        } catch (OdisException e) {
            if (LOGGER.isLoggable(BasicLevel.ERROR)) {
                LOGGER.log(BasicLevel.ERROR, e.getMessage());
            }
        } catch (SQLException e) {
            throw e;
        } finally {
            try {
                SqlConnectionHandler.freeConnection(conn);
            } catch (Exception e) {
            }
        }
    }

    /**
     * Execute.
     * @param conn the conn
     * @param query the query
     * @throws OdisAttributException the odis attribut exception
     * @throws SQLException the SQL exception
     */
    protected static void execute(Connection conn, String query)
            throws OdisAttributeException, SQLException {
        execute(conn, query, null);
    }

    /**
     * Execute.
     * @param conn the conn
     * @param query the query
     * @param parameters the parameters
     * @throws OdisAttributException the odis attribut exception
     * @throws SQLException the SQL exception
     */
    protected static void execute(Connection conn, String query,
            StatementParameterBean[] parameters) throws OdisAttributeException,
            SQLException {
        PreparedStatement pstmt = conn.prepareStatement(query);
        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
            LOGGER.log(BasicLevel.DEBUG, query);
        }
        loadParameter(parameters, pstmt);
        pstmt.execute();
    }

    /**
     * Execute query.
     * @param conn the conn
     * @param query the query
     * @param parameters the parameters
     * @return the result set
     * @throws OdisAttributException the odis attribut exception
     * @throws SQLException the SQL exception
     */
    protected static ResultSet executeQuery(Connection conn, String query,
            StatementParameterBean[] parameters) throws OdisAttributeException,
            SQLException {
        ResultSet result = null;
        PreparedStatement pstmt = conn.prepareStatement(query);
        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
            LOGGER.log(BasicLevel.DEBUG, query);
        }
        loadParameter(parameters, pstmt);
        result = pstmt.executeQuery();
        listPreparedStatement.put(pstmt, new Date());
        return result;
    }

    /**
     * Load parameter.
     * @param parameters the parameters
     * @param pstmt the pstmt
     * @throws OdisAttributException the odis attribut exception
     * @throws SQLException the SQL exception
     */
    private static void loadParameter(StatementParameterBean[] parameters,
            PreparedStatement pstmt) throws OdisAttributeException, SQLException {
        if (parameters != null) {
            for (int i = 0; i < parameters.length; i++) {
                StatementParameterBean parameter = parameters[i];
                parameter.setInStatement(pstmt, i + 1);
            }
        }
    }

    /**
     * Close all old (older than statementAge) opened prepared statement.
     * 
     * @param statementAge the statement age
     */    
    public static void closeAllOpenedPreparedStatement(long statementAge) {
    	long currentTimestamp = new Date().getTime();
    	synchronized (listPreparedStatement) {
    		Object[] pstmList = listPreparedStatement.keySet().toArray();
    		for (int i = 0; i < pstmList.length; i++) {
    			PreparedStatement psmt = (PreparedStatement) pstmList[i];
				Date date = (Date) listPreparedStatement.get(psmt);
				if (date.getTime() - currentTimestamp < statementAge) { 
					try {
						psmt.close();
					} catch (SQLException e) {
						e.printStackTrace();
					}
					listPreparedStatement.remove(psmt);
				}
			}
    	}
    }
}
